@@ -106,8 +106,8 @@ static char * php_zip_make_relative_path(char *path, size_t path_len) /* {{{ */
 		return NULL;
 	}
 
-	if (IS_ABSOLUTE_PATH(path, path_len)) {
-		return path + COPY_WHEN_ABSOLUTE(path) + 1;
+	if (IS_SLASH(path[0])) {
+		return path + 1;
 	}
 
 	i = path_len;
@@ -606,9 +606,8 @@ int php_zip_glob(char *pattern, int pattern_len, zend_long flags, zval *return_v
 		add_next_index_string(return_value, globbuf.gl_pathv[n]+cwd_skip);
 	}
 
-	ret = globbuf.gl_pathc;
 	globfree(&globbuf);
-	return ret;
+	return globbuf.gl_pathc;
 #else
 	zend_throw_error(NULL, "Glob support is not available");
 	return 0;
@@ -1481,21 +1480,6 @@ static ZIPARCHIVE_METHOD(open)
 		ze_obj->filename = NULL;
 	}
 
-#if LIBZIP_VERSION_MAJOR > 1 || LIBZIP_VERSION_MAJOR == 1 && LIBZIP_VERSION_MINOR >= 6
-	/* reduce BC break introduce in libzip 1.6.0
-	   "Do not accept empty files as valid zip archives any longer" */
-
-	/* open for write without option to empty the archive */
-	if ((flags & (ZIP_TRUNCATE | ZIP_RDONLY)) == 0) {
-		zend_stat_t st;
-
-		/* exists and is empty */
-		if (VCWD_STAT(resolved_path, &st) == 0 && st.st_size == 0) {
-			flags |= ZIP_TRUNCATE;
-		}
-	}
-#endif
-
 	intern = zip_open(resolved_path, flags, &err);
 	if (!intern || err) {
 		efree(resolved_path);
@@ -1840,8 +1824,7 @@ static ZIPARCHIVE_METHOD(addFile)
 		entry_name_len = ZSTR_LEN(filename);
 	}
 
-	if (php_zip_add_file(intern, ZSTR_VAL(filename), ZSTR_LEN(filename),
-			entry_name, entry_name_len, offset_start, offset_len) < 0) {
+	if (php_zip_add_file(intern, ZSTR_VAL(filename), ZSTR_LEN(filename), entry_name, entry_name_len, 0, 0) < 0) {
 		RETURN_FALSE;
 	} else {
 		RETURN_TRUE;
